<?php
	/* $Id: eDGetters.php,v 1.2 2002/08/12 11:33:17 bpotier Exp $ */
	require('eDQuery.php');
	require('eDMySQLMS.php');

	/**
	 * Generic Record Getter
	 *
	 * This script automatically builds the query to retrieve the values of records of a master table and of its child tables
	 * Child tables are detected based on the master table foreign keys.
	 * A where clause can be provided to the edGet function to restrict the search.
	 * This is a intermediary version waiting for the implementation of eDObjects, a concept that will greatly simplify this script
	 *
	 * @version eDGetters.php v1.0 4/09/01
	 * @author	Bertrand Potier <bpotier@edreamers.org>
	 * @package	eDFramework

	 */
	class eDGetters {
		var $debug; 
		var $arr_tableFields;
		var $arr_dbTables;
		var $MySQLMS;

		
		/**
		 * Class Constructor
		 * 
		 * @param	boolean	$debug			enter function in debug mode (enables outputs) or not
		 */
		function eDGetters($debug) {
			$this->debug = $debug;
			// Build a new description of all databases, tables and fields on the default MySQL server
			$this->MySQLMS = new eDMySQLMS($this->debug);
			$this->MySQLMS->buildSchema(null, null);
			// retrieves the list of tables-fields couples
			$this->arr_tableFields = $this->MySQLMS->get_tableFields();
			// retrieves the list of databases-tables couples
			$this->arr_dbTables = $this->MySQLMS->get_dbTables();
		}


		/**
		 * Retrieves the records from Master and child tables
		 *
		 * @param	string	$database	name of the database on which to perform the search
		 * @param	string	$table		name of the master table
		 * @param	string	$mode		search in 'normal'(master table only) or 'extended' (child tables also) mode
		 * @param	array	$arr_where	array describing the where clause, ex: array('systemid' => "'1'") or array("systemid > '1'" => '') for clause not based on = operators
		 * @param	string	$orderBy	name of the field(s) to use to order the results of the query
		 * @return	array				array of result records
		 */
		function eDGet($database, $table, $mode, $arr_where, $orderBy) {
			echo '<br><br>Entering eDGet! - '.$table;

			$fields = $this->arr_tableFields[$table];

			$arr_select = $fields;
			$arr_from = array($database.'.'.$table);

			switch($mode) {
				case 'normal':
					break;
				case 'extended':
					for($i=0; $i < sizeof($fields); $i++) {
						$str_ = $fields[$i];
						echo '<br><strong>eDGet Field</strong>: '.$str_; 

						// Detect Foreign keys based on field name terminology (ends with 'id')
						if ( ($table.'id' != $str_) && ( ereg('id$', $str_) ) ) {
							// Retrieve Foreign table name based on field name terminology
							$foreignTable = substr($str_, 2, strlen($str_)-4);
							if ($this->debug) { echo ' from table '.$foreignTable; } 
							
							// Retrieve the name of the db in which the table is contained
							$arr_rst = $this->MySQLMS->search('table', $foreignTable);
							// Update the WHERE, FROM and SELECT clauses
							$arr_from = array_merge($arr_from, array($arr_rst[0].'.'.$foreignTable) );
							$arr_select = array_merge($arr_select, $this->arr_tableFields[$foreignTable] );
							$arr_where = array_merge($arr_where, array($foreignTable.'id' => $str_) );
						}
						unset($foreignTable);
					}
					break;
				default:
					break;
			}

			if ($this->debug) { echo '<br>'; }
			$myQuery_ = new eDQuery($database, $arr_from, $arr_select, $arr_where, $orderBy, $this->debug);
			return $myQuery_->getRecords();
		}
	}
?>