<?php
// $Id$
require_once(dirname(__FILE__).'/'.'eDUtilities.php');

/**
 * MySQL Management System - DB Schema Builder
 *
 * This class purpose is to build a complete description of the databases, tables and fields
 * of a MySQL server
 *
 * @access	public
 * @author	Bertrand Potier <bpotier@edreamers.org>
 * @package	eDFramework
 */
class eDMySQLMS extends eDUtilities {
	/**
	 * Associative array of tables as keys and fields as values
	 * @var		array $arr_tableFields
	 * @access	private
	 */
	var $arr_tableFields = array();
	
	/**
	 * Associative array of dbs as keys and tables as values
	 * @var		array $arr_dbTables
	 * @access	private
	 */		
	var $arr_dbTables = array();

	/**
	 * Associative array of fields with their properties
	 * @var		array $arr_fieldsProperties
	 * @access	private
	 */
	var $arr_fieldsProperties = array();

	/**
	 * a DB to analyse
	 * @var		string		$aDB
	 * @access	private
	 */
	 var $aDB = '';

	/**
	 * a Table to analyse
	 * @var		string		$aTable
	 * @access	private
	 */
	 var $aTable = '';

	/**
	 * a Host to connect to
	 * @since		31/12/2002	BPO	SFBG#660339	eDFramework - eDMySQLMS Database Connect
	 * @var			string		$aHost
	 * @access	private
	 */
	 var $aHost = '';

	/**
	 * a User to connect the database server
	 * @since		31/12/2002	BPO	SFBG#660339	eDFramework - eDMySQLMS Database Connect
	 * @var			string		$aUser
	 * @access	private
	 */
	 var $aUser = '';

	/**
	 * a Password for user $aUser
	 * @since		31/12/2002	BPO	SFBG#660339	eDFramework - eDMySQLMS Database Connect
	 * @var			string		$aPassword
	 * @access	private
	 */
	 var $aPassword= '';


	/**
	 * Class Constructor
	 *
	 * Change Log
	 * 31/12/2002	BPO	SFBG#660339	eDFramework - eDMySQLMS Database Connect
	 *
	 * @param		string	$mode					set execution mode to basic (contructor only), advanced (build the db schema) or extended (same as advanced)
	 * @param		boolean	$traceEnabled	set trace mode (enables outputs) to TRUE or FALSE
	 */
	function eDMySQLMS(	
		$connId = null, 
		$aDB, 
		$aTable, 
		$mode = 'extended',
		$traceEnabled = FALSE ) {

		$this->setTrace( $traceEnabled );
		$this->trace(__FILE__.' :: __START__ :: Function eDMySQLMS Constructor');
		$this->aDB = $aDB;
		$this->aTable = $aTable;
		
		// Testing connection id, if null then mode is overwritten to 'basic'
		if ( $connId == null ) { 
			$mode = 'basic';
			$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function eDMySQLMS - no valid MySQL connection id, mode is set to basic: Provide a connection id or make an explicit call to the connect() function!' );
		} else {
			$this->connId = $connId;
		}
		
		switch($mode) {
			case 'extended':
			case 'advanced':
				$this->buildSchema();
				break;
			case 'basic':
				break;
			default:
				break;
		}
	}


	/**
	 * Connect the host database
	 */
	 function connect(
		 $dbHost = 'localhost',
		 $dbUser ='root',
		 $dbPassword ='') {
		 
		 $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function connect :: User '.$dbUser.' attempting connection to '.$dbHost);
		 if ( $connId = mysql_connect($dbHost, $dbUser, $dbPassword) ) {
			 $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function connect :: Connection to db successful! ('.$connId.')');
			 return $connId;
			} else {
				$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function runQuery :: Connection to db failed! ('.mysql_error().')');
				return FALSE;
			}
	 }


	/**
	 * Build Schema of dbs, tables and fields on local MySQL Server 
	 *
	 * Change Log
	 * 31/12/2002	BPO	SFBG#660339	eDFramework - eDMySQLMS Database Connect
	 */
	function buildSchema() {
		$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema - '.$this->aDB.' - '.$this->aTable);
		$atable = $this->aTable;
		
		// Checking Parameters
		if ( !isset($this->aDB) && isset($this->aTable) ) {
			$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema :: Missing db name when table name is set, Execution stopped!');
			exit();
		} else if ( isset($this->aDB) && !$this->is_defined($this->aTable) ) {
			$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema :: db name only is set, retrieving tables and fields list only');
			$this->buildTablesList($this->aDB);
		} else if ( isset($this->aDB) && isset($this->aTable) ) {
			$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema :: db and table name set, retrieving fields information only');
			$this->buildFieldsList($this->aDB.'.'.$this->aTable);
		} else {
			$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema :: No DB or Table name provided, building full schema!');
			$dbs = mysql_list_dbs($this->connId) or die( $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema :: DB Listing failed! ('.mysql_error().')') ) ;
			$num_dbs = mysql_num_rows($dbs) or die( $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema :: Getting number of rows failed! ('.mysql_error().')') ) ;;

			for($i=0; $i<$num_dbs; $i++) {
				$arr_dbs[$i] = mysql_db_name($dbs, $i) or die( $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema :: Getting db name failed! ('.mysql_error().')') ) ;;
				$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildSchema ::  current DB is '.$arr_dbs[$i]);
				$this->buildTablesList($arr_dbs[$i]) ;
			}
		}
	}


	/**
	 * Build the list of tables in a db
	 *
	 * @param	string	$adb	name of the db to process
	 */
	function buildTablesList( $adb ) {
		$this->trace(__FILE__.'::Line '.__LINE__.'::'.'buildTablesList - '.$adb);

		$tables = mysql_list_tables( $adb, $this->connId) or die( $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildTablesList :: Tables Listing failed! ('.mysql_error().')') );
		$num_tables = mysql_num_rows( $tables ) or die( $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildTablesList :: Getting number of rows failed! ('.mysql_error().')') );

		for( $j = 0; $j < $num_tables; $j++ ) {
			$arr_tables[$j] = mysql_tablename($tables, $j) or die( $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildTablesList :: Getting table name failed! ('.mysql_error().')') );
			$this->buildFieldsList( $adb.'.'.$arr_tables[$j] );
		}
		$this->arr_dbTables = array_merge( $this->arr_dbTables, array( $adb => $arr_tables ) );
		unset($arr_tables);
	}


	/**
	 * Build the list of fields in table
	 *
	 * @param	string	$adb	name of the db to process
	 * @param	string	$atable	name of the table to process
	 */
	function buildFieldsList(
		$atable,
		$getProperties = TRUE ) {
		$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.' Function buildFieldsList - '.$atable);

		$arr_fields = array();

		$qry_ = 'SHOW FIELDS FROM '.$atable;
		$fields = mysql_query($qry_, $this->connId);

		while( $row = mysql_fetch_array($fields, MYSQL_ASSOC) ) {
			$this->trace(__FILE__.'::Line '.__LINE__.'::'.' FIELDS: '.$row['Field'].' having value "'.$row.'"');
			if ($getProperties) { 
				$this->arr_fieldsProperties = array_merge($this->arr_fieldsProperties, array($row['Field'] => $row) ); 
			}
			$arr_fields = array_merge($arr_fields, array($row['Field']) );
		}
		
		$this->arr_tableFields = array_merge( $this->arr_tableFields, array( $atable => $arr_fields ) );
		unset($arr_fields);
		mysql_free_result($fields);
	}


	/**
	 * Search the presence of field, table or db on the server
	 *
	 * @param	string	$objectType		the type of the object to search, either 'database','table' or 'field'
	 * @param	string	$objectName		the name of the object to find (a db, table or field name)
	 * @return	array					array[0]<-db, array[1]<-table, array[2]<-field or null if not found
	 */
	function search($objectType, $objectName) {
		$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function search - '.$objectType.' - '.$objectName);

		$arr_ = array(null, null, null);

		switch($objectType){
			case 'database':
				if( in_array($objectName, $this->arr_dbTables) ) {
					$arr_ = array($objectName, null, null);
					return $arr_;
					break;
				}
				return $arr_;
				break;
			case 'table':
				foreach( $this->arr_dbTables as $db => $arr_tables) {
					
					$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.' Searched in db '.$db.' containing '.sizeof($arr_tables).' tables');
					
					if( in_array($objectName, $arr_tables) ) {
						$this->outl(' - FOUND in '.$db.'!');
						$arr_ = array($db, $objectName, null );
						return $arr_; // Exit as soon as a first good result is found
						break;
					}
				}
				return $arr_;
				break;
			case 'field':
				foreach( $this->arr_tableFields as $table => $arr_fields) {
					if( in_array($objectName, $arr_fields) ) {
						$arr_ = array(null, $table, $objectName);
						foreach( $this->arr_dbTables as $db => $arr_tables) {
							if( in_array($objectName, $arr_tables) ) {
								$arr_ = array($db, $table, $objectName);
								return $arr_;
								break;
							}
						}
					}
				}
				return $arr_;
				break;
			default:
				break;
		}
	}


	/**
	 * Get array of kv pairs of tables and fields
	 *
	 * Keys are tables names associated to arrays storing the list of fields in related table
	 *
	 * @return	array	array of kv pairs of tables and fields
	 */
	function getTableFields() {
		$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.' Function getTableFields');
		return $this->arr_tableFields;
	}


	/**
	 * Get array of kv pairs of databases and tables
	 *
	 * Keys are databases names associated to arrays storing the list of tables in related database
	 *
	 * @return	array	array of kv pairs of databases and tables
	 */

	function getDBTables() {
		$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.' Function getDBTables');
		return $this->arr_dbTables;
	}

	
	/**
	 * Get array of kv pairs of fields and their properties
	 *
	 * Keys are fields names associated to arrays storing the list of their properties
	 *
	 * @return	array	array of kv pairs of fields and their properties
	 * @since	eDFramework 2.1
	 */

	function getFieldsProperties() {
		$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.' Function getFieldsProperties');
		return $this->arr_fieldsProperties;
	}
}
?>