<?php
	require_once(dirname(__FILE__).'/'.'eDUtilities.php');
	require_once(dirname(__FILE__).'/'.'eDMySQLMS.php');
	require_once(dirname(__FILE__).'/'.'eDQuery.php');
	/**
	 * 'eDObject - eDreamers Object'
	 *
	 * eDObject wraps database tables into objects that can be manipulated through
	 * your PHP scripts. eDObject are composed of a unique master table, 1 to many 
	 * child tables linked to the master table using keys and constraints on those keys.
	 * 
	 * The definition of an eDObject relies on the records of three tables that you 
	 * have to add to each database for which you'd like to implement eDObjects.
	 *
	 * $Id: eDObject.php,v 1.1.1.1 2002/12/30 13:49:08 bpotier Exp $ 
	 *
	 * @access	public
	 * @version v1.0 22/02/2002
	 * @author	Bertrand Potier <bpotier@edreamers.org>
	 * @package	eDFramework	 
	 */
	class eDObject extends eDUtilities {
		var $arr_objectFields = array();
		var $arr_objectKeys = array();
		var $arr_objectConstraints = array();
		var $arr_objectSelect = array();
		var $arr_objectFrom = array();
		var $arr_objectWhere = array();
		var $tbl_objectMasterTable = '';
		var $tbl_objectMasterTableSimple = '';

		var $db_objectMasterDB = '';
		var $objectInstanceId = null;
		var $objectInstanceName = '';

		var $_eDMySQLMS;
		
		// xeDObject Tables Definition
		var $tbl_xedkey = 'xedkey';
		var $tbl_xedobject = 'xedobject';
		var $tbl_xedconstraint = 'xedconstraint';

		// xeDObject Fields Definition
		var $fld_xedobjectName = 'xtname';
		var $fld_xedobjectMasterDB = 'xtmasterdb';
		var $fld_xedobjectMasterTable = 'xtmastertable';
		var $fld_xedkeyObjectId = 'xyobjectid';
		var $fld_xedkeyMasterField = 'xymasterfield';
		var $fld_xedkeyChildDB = 'xychilddb';
		var $fld_xedkeyChildTable = 'xychildtable';
		var $fld_xedkeyChildField = 'xychildfield';
		var $fld_xedconstraintKeyId = 'xtkeyid';

		var $arr_keyQueries = array(); // Queries to retrieve key instance(s)' values

		/**
		 *  eDObject STRUCTURE - Object Constructor
		 *
		 * @param	 @mode	 basic, extended
		 */
		function eDObject( $objectDatabase, $objectId, $mode, $objectInstanceId, 
			$trace = FALSE, 
			$traceMode = 'log' ){
		
			$this->setTrace( $debug, $traceMode );
			$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function eDObject', 'line');

			$this->objectInstanceId = $objectInstanceId;
			
			// Create a new instance of the eDMySQLMS class
			$this->_eDMySQLMS = new eDMySQLMS('normal', null, null, $connId, $debug);

			// Build the list of fields in the object table
			$this->_eDMySQLMS->buildFieldsList($objectDatabase.'.'.$this->tbl_xedobject, false);
			
			// Retrieve the list of fields with their values
			$this->arr_objectFields = $this->_eDMySQLMS->getTableFields();
			$arr_where = array($this->tbl_xedobject.'id' => "'".$objectId."'");
			$arr_from = array($objectDatabase.'.'.$this->tbl_xedobject);
			$arr_select = $this->arr_objectFields[$objectDatabase.'.'.$this->tbl_xedobject];
			$myeDQuery_ = new eDQuery($objectDatabase, $arr_from, $arr_select, $arr_where, null, 'extended', $debug);
			$arr_objectFieldsValue = $myeDQuery_->getRecords();
			$this->objectInstanceName = $arr_objectFieldsValue[0][$this->fld_xedobjectName]; 
			$this->db_objectMasterDB = $arr_objectFieldsValue[0][$this->fld_xedobjectMasterDB];
			$this->tbl_objectMasterTable = $this->db_objectMasterDB.'.'.$arr_objectFieldsValue[0][$this->fld_xedobjectMasterTable];
			$this->tbl_objectMasterTableSimple = $arr_objectFieldsValue[0][$this->fld_xedobjectMasterTable];

			switch($mode) {
				case 'basic':
					$this->buildKeys($objectDatabase, $objectId);
					$this->runKeyQuery();
					$this->buildConstraints($objectDatabase, $objectId);
					break;
				case 'extended':
					$this->buildKeys($objectDatabase, $objectId);
					$this->buildConstraints($objectDatabase, $objectId);
					$this->buildObjectQuery();
					$this->runObjectQuery();
					break;
				default:
					break;
			}
		}


		/**
		 * eDObject STRUCTURE
		 */
		function buildKeys($objectDatabase, $objectId) {
			
			$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function buildKeys', 'line');

			$arr_select = array('xedkeyid', 'xymasterfield','xychilddb','xychildtable','xychildfield');
			$arr_where = array( $this->fld_xedkeyObjectId => $objectId);
			$qry_ = new eDQuery($objectDatabase, array($this->tbl_xedkey), $arr_select, $arr_where, null, 'extended', $this->debug);
			$arr_objectKeys = $qry_->getRecords();

			$i = sizeof($arr_objectKeys);
			$arr_search = array();

			foreach ($arr_objectKeys as $key => $value) {
				$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function buildKeys :: '.$key.'-'.$value[$this->fld_xedkeyMasterField].' :: '.$arr_objectKeys[$key][$this->fld_xedkeyMasterField], 'line');

				$k = array_search($value[$this->fld_xedkeyMasterField], $arr_search);

				if ( $k === null ) {
					$this->trace(__FILE__.'::Line '.__LINE__.'::'.'Function buildKeys::'.' key '.$value[$this->fld_xedkeyMasterField].' is not in array', 'line');
					$arr_search = array_merge($arr_search, array($value[$this->fld_xedkeyMasterField]));
					$this->arr_objectKeys[$key]['xedkeyid'] = $value['xedkeyid'];
					$this->arr_objectKeys[$key][$this->fld_xedkeyMasterField] = $value[$this->fld_xedkeyMasterField];
					$this->arr_objectKeys[$key][$this->fld_xedkeyChildDB] = $value[$this->fld_xedkeyChildDB];
					$this->arr_objectKeys[$key][$this->fld_xedkeyChildTable] = $value[$this->fld_xedkeyChildTable];
					$this->arr_objectKeys[$key][$this->fld_xedkeyChildField] = $value[$this->fld_xedkeyChildField];
					$this->arr_objectKeys[$key]['select'] = array($value[$this->fld_xedkeyChildTable].'.'.$value[$this->fld_xedkeyChildField]);
					$this->arr_objectKeys[$key]['from'] = array($value[$this->fld_xedkeyChildDB].'.'.$value[$this->fld_xedkeyChildTable]);
					$this->arr_objectKeys[$key]['where'] = null;
				} else {
					$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function buildKeys :: '.' key '.$value[$this->fld_xedkeyMasterField].' is already in array', 'line');
					// Checking if child table is already in FROM statement, if no adding it
					$j = array_search($value[$this->fld_xedkeyChildDB].'.'.$value[$this->fld_xedkeyChildTable], $this->arr_objectKeys[$k]['from']);
					if ( !$j ) {
						$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function buildKeys :: '.' table '.$value[$this->fld_xedkeyChildTable].' is not in FROM statement', 'line');
						$this->arr_objectKeys[$k]['from'] = array_merge($this->arr_objectKeys[$k]['from'], array($value[$this->fld_xedkeyChildDB].'.'.$value[$this->fld_xedkeyChildTable]) );
					} else {
						$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function buildKeys :: '.' table '.$value[$this->fld_xedkeyChildTable].' is in FROM statement', 'line');
					}
					$this->arr_objectKeys[$k]['select'] = array_merge($this->arr_objectKeys[$k]['select'], array($value[$this->fld_xedkeyChildTable].'.'.$value[$this->fld_xedkeyChildField]) );
				}
			}
		}


		/**
		 * eDObject STRUCTURE
		 */
		function buildConstraints($objectDatabase, $objectId){

			$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function buildConstraints', 'line');

			if ( sizeof( $this->arr_objectKeys ) ) {
				foreach ($this->arr_objectKeys as $key => $value_) {
					$arr_select = array('xttable','xtfield', 'xtoperator', 'xtvalue');
					$arr_where = array( $this->fld_xedconstraintKeyId => $value_['xedkeyid']);
					$qry_ = new eDQuery($objectDatabase, array($this->tbl_xedconstraint), $arr_select, $arr_where, null, 'extended', $this->debug);
					$this->arr_objectConstraints = $qry_->getRecords();
				}
			}
		}


		/**
		 * eDObject CONTENT
		 */
		function buildObjectQuery(){

			$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.' Function buildObjectFields', 'line');

			if ( sizeof( $this->arr_objectKeys ) ) {
				$arr_ = $this->getMasterFieldsName();
				$this->arr_objectSelect = $arr_;
				$this->arr_objectFrom = $this->tbl_objectMasterTable;
				
				foreach ($this->arr_objectKeys as $key => $value_) {
					for ($i=0; $i<sizeof($arr_); $i++) {
						if ($arr_[$i] == $value_[$this->fld_xedkeyMasterField]) {
							$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function buildObjectFields - Found a key on '.$arr_[$i], 'line');
							$this->arr_objectSelect = array_merge($this->arr_objectSelect, $value_['select']);
							
							// FROM Statement
							if ( ! @in_array($value_[$this->fld_xedkeyChildDB].'.'.$value_[$this->fld_xedkeyChildTable], $this->arr_objectFrom) ) {
								$this->arr_objectFrom = array_merge($this->arr_objectFrom, $value_['from']);
							}
							
							// WHERE Statement
							$this->arr_objectWhere = array_merge($this->arr_objectWhere, array($arr_[$i] => $value_[$this->fld_xedkeyChildTable].'id') );
						}
					}
				}

				// WHERE Statement - Adding the Instance information if any
				if ($this->objectInstanceId) {
					$str_ .= '(';
					if (is_array($this->objectInstanceId)) {
						echo 'isarray -';
						for($i=0; $i<sizeof($this->objectInstanceId); $i++) {
							if($i>0 && $i<sizeof($this->objectInstanceId)) {
								$str_ .= ' OR ';
							}
							$str_ .= $this->tbl_objectMasterTableSimple."id = '".$this->objectInstanceId[$i]."'";
						}
					} else if (is_int($this->objectInstanceId)) {
						$str_ = $this->tbl_objectMasterTable."id = '".$this->objectInstanceId."'";
					}
					$str_ .= ')';
					$this->arr_objectWhere = array_merge($this->arr_objectWhere, array($str_ => '') );
				}
			}
		}


		
		/**
		 * eDObject CONTENT
		 */
		function runObjectQuery() {
			$this->trace(__FILE__.' :: Line '.__LINE__.' :: Function runObjectQuery', 'line');

			$myeDQuery = new eDQuery($this->db_objectMasterDB, $this->arr_objectFrom, $this->arr_objectSelect, $this->arr_objectWhere, null, 'extended', $this->debug);
			$this->arr_objectRecords = $myeDQuery->getRecords();
		}


		/**
		 * eDObject CONTENT
		 */
		function getObjectRecords(){
			return $this->arr_objectRecords;
		}

		
		/**
		 * eDObject CONTENT
		 */
		function runKeyQuery(){
			foreach($this->arr_objectKeys as $key => $value) {
				$eDQuery_ = new eDQuery($this->db_objectMasterDB, $value['from'], $value['select'], $value['where'], $value['select'][0], 'extended', $this->debug);
				$this->arr_objectKeys[$key]['records'] = $eDQuery_->getRecords();
			}
		}

		/**
		 * eDObject CONTENT
		 */
		function getKeyQueries() {
			return $this->arr_keyQueries;
		}

		/**
		 * eDObject STRUCTURE
		 */
		function getMasterFieldsName(){
			$this->_eDMySQLMS->buildFieldsList($this->tbl_objectMasterTable, true);
			$arr_ = $this->_eDMySQLMS->getTableFields();
			return $arr_[$this->tbl_objectMasterTable];
		}


		/**
		 * eDObject STRUCTURE
		 */
		function getObjectFieldsName() {
			return $this->arr_objectSelect;
		}


		/**
		 * eDObject STRUCTURE
		 */
		function getMasterTable() {
			return $this->tbl_objectMasterTable;
		}
		

		/**
		 * eDObject STRUCTURE :: Returns the Object keys
		 * 
		 */
		function getKeys(){
			return $this->arr_objectKeys;
		}

		/**
		 * eDObject STRUCTURE :: Returns TRUE if $field is an eDObject's key
		 * 
		 */
		function isKey($aField, $getKey){
			foreach ($this->arr_objectKeys as $i => $key) {
				$this->trace(__FILE__.' :: Line '.__LINE__.' :: '.'Function isKeys :: '.$aField, 'line');

				$j = array_search($aField, $key);

				if ( $j === null ) {
					return FALSE;
				} else {
					if ($getKey) {
						return $key;
					} else {
						return TRUE;
					}
				}
			}
		}

		
		/**
		 * eDObject STRUCTURE
		 */		
		function getConstraints(){
			 return $this->arr_objectConstraint;
		}

		/**
		 * eDObject eDMySQLMS
		 */
		 function getMySQLMS() {
			return $this->_eDMySQLMS;
		 }

		/**
		 * Returns Object Name
		 */
		function getObjectName() {
			return $this->objectInstanceName;
		}

		/**
		 * Find Object : Returns objectid if an object is found
 		 */
		function findObject($objectdb, $masterTable){
			$arr_select = array('xedobjectid');
			$arr_from = array($objectdb.'.xedobject');
			$arr_where = array('xtmastertable' => "'".$masterTable."'");
			$aQuery = new eDQuery($objectdb, $arr_from, $arr_select, $arr_where, null, 'extended', $this->debug);
			$arr_objectids = $aQuery->getRecords();

			if(sizeof($arr_objectids)) {
				return $arr_objectids[0]['xedobjectid'];
			}
		}
		
	}
?>