<?php
require_once dirname(__FILE__).'/'.'eDUtilities.php';

/**
 * MySQL Select Query Generator
 *
 * This script centralizes all the code required to build and launch a Select query aindex.phpgainst a MySQL db
 * eDQuery can build all kind of select queries but assumes that the most encoutered one are build on the
 * following model: SELECT field1 [, field2] FROM database1.table1 [, database1.table2] WHERE field3 = 'value'
 * This would be translated into the following using eDQuery:
 *	$myQuery = new eDQuery('database1', array('table1', [table2]), array('field1','field2'), array('field3' => "'value'"), null, $debug );
 *	$arr_results = $myQuery->getRecords();
 * Another version is under construction supporting all types of PHP supported databases
 *
 * Why and When should I use eDQuery?
 * ----------------------------------------------------
 * First because it will centralize into one script only the code to access your MySQL db.
 * It gives more independancy to your script towards your db and all improvments made to eDQuery
 * will be available to all your scripts without having to rewrite your code. Just imagine that
 * you'd like to use an Oracle db instead of a MySQL one, you'd only need to make minor changes to
 * this script (mainly in the runQuery() and getRecords() functions.
 * eDQuery also provides useful functions like getValue to easily find a field value in the array of your records
 * So, when to use it? As often as possible of course :)
 *
 * @author		Bertrand Potier <bpotier@edreamers.org>
 * @package		eDFramework
 * @see			eDUtilities
 * @link		http://eDreamers.org
 * @version     1/02/2004 - v2.21
 */
class eDQuery extends eDUtilities {
    /**
     * eDQuery::$dbName
     *
     * { Description }
     */
    var $dbName = '';			// name of the database to run the query

    /**
     * eDQuery::$arr_from
     *
     * { Description }
     */
    var $arr_from = array();	// name of the table

    /**
     * eDQuery::$arr_select
     *
     * { Description }
     */
    var $arr_select = array();	// array containing exact name of the field used for the select clause

    /**
     * eDQuery::$where
     *
     * { Description }
     */
    var $where = '';			// array containing field name and associated value(s) to be used in the where clause
 
     /**
      * eDQuery::$orderBy
      *
      * { Description }
      */
     var $orderBy = '';			// field name to used with order by clause

     /**
      * eDQuery::$qry_
      *
      * { Description }
      */
     var $qry_ = '';				// the sql query
     
     /**
      * eDQuery::$rst_
      *
      * { Description }
      */
     var $rst_ = '';				// the result of the sql query

     /**
      * eDQuery::$arr_rec
      *
      * { Description }
      */
     var $arr_rec = array();		// array containing the formatted results of query

     /**
      * eDQuery::$traceMode
      *
      * { Description }
      */
     var $traceMode = 'log';		// trace mode, either screen or log
     
     /**
      * eDQuery::$connId
      *
      * { Description }
      */
     var $connId = 0;			// MySQL connection id


    /**
	 * Class Constructor
	 *
	 * Today the constructor launches the construction and the execution of the query (extended mode) but
	 * this can be moved away to stay under the responsability of the script calling eDQuery
	 *
     * Change Log
     * 15/08/2003   BPO SFFRxxxxxx eDFramework - eDQuery EXTRA Statements
     *
	 * @param		string		$dbName		name of the db against which the query will be launched
	 * @param		array		$arr_from	array containing the list of table to query, ex: array('system', 'employee')
	 * @param		array		$arr_select	array containing the list of fields to retrieve, ex: array('systemid', 'employeeid')
	 * @param		array		$arr_where	array describing the where clause, ex: array('systemid' => "'1'") or array("systemid > '1'" => '') for clause not based on = operators
     * @param	string		$orderBy	    name of the field(s) to use to order the results of the query
     * @param	string		$mode		    only build the query in normal mode, build and run the query in extended mode
     * @param	boolean	    $traceEnabled	enter function in debug mode (enables outputs) or not
     * @param   string      $extra          extra SQL instructions to add to the query, to be used for other than SELECT, WHERE, ORDER BY instructions
	 */
	function eDQuery(
		$connId = null,
		$dbName,
		$arr_from,
		$arr_select,
		$where = '',
		$orderBy = '',
		$mode = 'extended',
		$trace = false,
        $str_extra = null,
        $limit = null,
        $traceMode = 'log',
        $traceFormat = 'text'
	) {
        $this->setTrace( $trace, $traceMode, $traceFormat, 'trace.log' );
    	$this->trace('eDQuery ::  __START__ :: Function eDQuery Constructor');

		$this->dbName = $dbName;
		$this->arr_from = $arr_from;
		$this->arr_select = $arr_select;
		$this->where = $where;
		$this->orderBy = $orderBy;

		// Testing connection id, if null then mode is overwritten to 'basic'
		if ( $connId == null ) {
			$mode = 'basic';
			$this->trace('eDQuery ::  Line '.__LINE__.' :: Function eDQuery - no valid MySQL connection id, mode is set to basic: Provide a connection id or make an explicit call to the connect() function!');
		} else {
			$this->connId = $connId;
		}

		switch($mode) {
			case 'basic':
				break;
			case 'advanced':
                $this->buildQuery( $str_extra );
				break;
			case 'extended':
                $this->buildQuery( $str_extra );
				$this->runQuery( $connId, $dbName );
				break;
			default:
				break;
		}
	}


    /**
     * Connect the host database
     *
     * { Description }
     *
     * @since eDQuery v2.0
     */
 	function connect(
        $dbHost = 'localhost',
        $dbUser ='root',
        $dbPassword =''
    ) {
		 $this->trace('eDQuery ::  Line '.__LINE__.' :: Function connect :: User '.$dbUser.' attempting connection to '.$dbHost, 'inline');
		 if ( $connId = mysql_connect($dbHost, $dbUser, $dbPassword) ) {
             $this->trace('eDQuery ::  Line '.__LINE__.' :: Function connect :: Connection to db successful! ('.$connId.')', 'inline');
             return $connId;
         } else {
             $this->trace('eDQuery ::  Line '.__LINE__.' :: Function runQuery :: Connection to db failed! ('.mysql_error().')', 'inline');
             return FALSE;
         }
    }


	/**
	 * Build the query
	 *
	 * Change Log
     * 15/08/2003	BPO	SFFR#xxxxxx eDFramework - eDQuery EXTRA Statements
     * 6/01/2003	BPO	SFFR#663119 eDFramework - eDQuery WHERE Statement
     * 31/12/2002	BPO	SFFR#591993	eDFramework - eDQuery From Statement
	 */
	function buildQuery( $str_extra ) {
        $this->trace('eDQuery ::  Line '.__LINE__.' :: Function buildQuery', 'inline');

		$this->qry_ = 'SELECT ';
		
		$i = sizeof($this->arr_select);
		foreach ($this->arr_select as $key => $field) {
			if ($key == $i-1) {	
				$this->qry_ .= $field.' ';
			} else {
				$this->qry_ .= $field.', ';
			}
		}

		// introducing the use of array for FROM clause
		$this->qry_ .= 'FROM ';
		$i = sizeof($this->arr_from);
		foreach ( $this->arr_from as $key => $table) {
			if ($key == $i-1) {	
					$this->qry_ .= $table;
				} else {
					$this->qry_ .= $table.', ';
				}
		}
        
        // SFFRxxxxxx
        $this->qry_ .= ' '.$str_extra;            
		if ( $this->where != null && $this->where != '' ) {
 			$this->qry_ .= ' WHERE '.$this->where;
		}

		// ORDER BY clause if not null
		if ( $this->orderBy != NULL && $this->orderBy != '' ) {
			$this->qry_ .= ' ORDER BY '.$this->orderBy;
		}
        
        // LIMIT if not null
        if ( isset( $limit ) ) {
            $this->qry_ .= ' LIMIT 0,'.$limit;
        }

		$this->trace('eDQuery ::  Line '.__LINE__.' :: Function buildQuery - Query '.$this->qry_.' has been built!', 'inline');
	}

	
	/**
	 * Run the query
	 *
	 * Change Log:
	 * 30/12/2002	BPO	SFBG#621819	eDFramework - eDQuery Debug Messages
	 * 16/12/2002	BPO	SFFR#591985 eDFramework - eDQuery MySQL Connect
	 */
	function runQuery( $connId, $dbName ) {
		$this->trace('eDQuery ::  Line '.__LINE__.' ::  Function runQuery', 'inline');

		mysql_select_db( $dbName, $connId ) or die( $this->trace('eDQuery ::  Line '.__LINE__.' :: Function runQuery :: Selecting db '.$dbName.' has failed! ('.mysql_error().')', 'inline', 'html') );
		
		$this->rst_ = mysql_query( $this->qry_, $connId ) or die( $this->trace('eDQuery ::  Line '.__LINE__.' :: Function runQuery :: Query '.$this->qry_.' is invalid!', 'inline', 'html') );
		
		$this->trace('eDQuery ::  Line '.__LINE__.' :: Function runQuery :: num rows is '.mysql_num_rows($this->rst_), 'inline');
		
		if ( ( mysql_num_rows( $this->rst_ ) != 0 ) ) {
			for ($i = 0; $i < mysql_num_rows( $this->rst_ ); $i++ ) {
				foreach ( $this->arr_select as $key => $field ) {
					$arr_rst[$field] = mysql_result( $this->rst_, $i, $key );
					$this->arr_rec[$i] = $arr_rst;
					$this->trace('eDQuery ::  Line '.__LINE__.' :: Function runQuery :: '.$field.' : '.$this->arr_rec[$i][$field], 'inline');
				}
			}
		} else if ( mysql_num_rows( $this->rst_) == 0 ) {
			$this->trace('eDQuery ::  Line '.__LINE__.' :: Function runQuery :: Query - '.$this->qry_.' - returned no result !', 'inline');
		} else {
			$this->trace('eDQuery ::  Line '.__LINE__.' :: Function runQuery :: Query - '.$this->qry_.' - is not valid ! ('.mysql_error().')', 'inline');
			exit();
		}
		mysql_free_result( $this->rst_ );
	}


    /**
     * Returns the generated query
     *
     * { Description }
     *
     * @return string the generated SQL query
     */
 	function getQuery() {
		return $this->qry_;
	}
	

	/**
	 * Returns two dimensional array of array containing one array per record
	 * results can be retrieved thanks to the following code in the calling script
	 * $arr_rec = $myeDQuery->getRecord();
	 * foreach ($arr_rec as $key => $value) {
	 *	echo $value['fieldname'];
	 * }
	 *
	 * ChangeLog:
	 * 30/12/2002	BPO	SFFR#659958 eDFramework - eDQuery getRecords Mode
	 *
	 * @return	array	the array of associative array, each representing a record returned by the query
	 */
	function getRecords($format = 'array') {
		$this->trace(__FILE__.'::Line '.__LINE__.'::'.'getRecords', 'inline');
		switch ($format) {
			case 'html':
				$str_ = '<ul>';
				foreach ($this->arr_rec as $key => $record) {
					foreach ($this->arr_select as $key => $field) {
						$str_ .= '<li>Field '.$field.' value is '.$record[$field].'</li>';
					}
				}
				$str_ .= '<ul>';
				return $str_;
				break;
			case 'text':
				$str_ = '';
				foreach ($this->arr_rec as $key => $record) {
					foreach ($this->arr_select as $key => $field) {
						$str_ .= '- Field '.$field.' value is '.$record[$field]."\n\r";
					}
				}
				return $str_;
				break;
			default:
				return $this->arr_rec;
				break;
		}
	}


	/**
	 * Finds a record within the array of records
	 *
	 * Example of use: $arr_foundRecord = getRecord('employee', 'Paul')
	 *
	 * Change Log
	 * 31/12/2002	BPO	SFFR#592005	eDFramework - eDQuery Search function
	 *
	 * @param	string	$searchKey		the name of field where to search
	 * @param	string	$searchValue	the field value to find
	 * @return	array	the FIRST record where 'searchValue' has  been found
	 */
	function getRecord( $searchKey, $searchValue ) {
		$this->trace(__FILE__.'::Line '.__LINE__.'::'.$searchKey.' - '.$searchValue.' - '.$returnKey, 'inline');
		foreach ($this->arr_rec as $id => $arr_record) {
			if ($arr_record[$searchKey] == $searchValue) { return $arr_record; }
		}
		return -1;
	}


	/**
	 * Finds a key within a record within the array of records
	 *
	 * Example of use: $findMe = getValue('employee', 'Paul', 'location')
	 *
	 * Change Log
	 * 31/12/2002	BPO	SFFR#592005	eDFramework - eDQuery Search function
	 *
	 * @param	string	$searchKey		the name of field where to search
	 * @param	string	$searchValue	the field value to find
	 * @param	string	$returnKey		the field from which to return the value once a record is found
	 * @return	string	the value of field 'returnKey' extracted from the record where 'searchValue' has first been found
	 */
	function getValue( $searchKey, $searchValue, $returnKey ) {
		$this->trace(__FILE__.'::Line '.__LINE__.'::'.$searchKey.' - '.$searchValue.' - '.$returnKey, 'inline');
		foreach ($this->arr_rec as $id => $arr_record) {
			if ($arr_record[$searchKey] == $searchValue) { return $arr_record[$returnKey]; }
		}
		return -1;
	}
}
?>
