<?php 
/**
 *  eDFramework Utilities Class (inspired by PHPDoc)
 *
 * As a kind of common base class, this class holds configuration values (e.g.
 * error handling) and provides useful methods (e.g. introspection()) to all 
 * kind of classes.
 * It does not have a constructor, so you can always inherit classes from this
 * class without any trouble.
 * 
 * Inspired from PHPDoc by Ulf Wendel <ulf.wendel@phpdoc.de>
 *
 * @package	eDFramework
 * @author	Bertrand Potier <support@edreamers.org>
 */
class eDUtilities {
	/**
	 * Boiler plates
	 * @see	trace()
	 */
	 var $CR_html = "<br>\n";
	 var $CR_text = "\n";

	/**
	 *
	 */
	var $traceEnabled = FALSE;

	/**
	 *
	 */
	var $mode = 'log';

	/**
	 *
	 */
	var $format = 'text';

	/**
	 *
	 */
	 var $str_file = 'trace.log';


	/**
	 * Sets the TRACE flag, mode and format
	 *
	 * If set to true the trace() will print messages sent to it. If no, the trace() function will do nothing.
	 * Use this function as an intelligent echo/print function to log, trace, debug the activity of our classes.
	 *
	 * @param		boolean	    $boo_				TRUE or FALSE to activate or de-activate the tracing
	 * @param		string		$str_mode	'screen' | 'log' to send trace to the screen or to a log file
 	 * @param		string		$str_format	'text' | 'html' to wrap trace message as text or html
 	 * @param		string		$str_file			log file relative pathname
	 * @return		1										on success
	 * @see			$trace, trace()
	 */
	function setTrace( 
		$boo_ = false, 
		$str_mode = 'log',
		$str_format = 'text',
		$str_file = 'trace.log') {

		$this->traceEnabled = $boo_;
		$this->mode = $str_mode;
		$this->format = $str_format;
		$this->str_file = $str_file;

		return 1;
	}

	

	/**
	* Print a string and flushes the output buffer
	*
	* Change Log
	* 2/01/2003	BPO	SFFR#661093	eDFramework - eDUtilities Trace Mode
	* - Function name has changed from out(...) to trace(...)
	* - Function parameters now have default values
	* - $mode parameter is now named $display
	* - $mode parameter is no used to set the output stream
	* - applying variable naming convention, adding variable type prefix to variable names
	*
	* @param	string 	$message								The message to print
	* @param	string 	$format			text | html			Display the message as text or HTML
	* @param	string 	$display			inline | block		Display the message inline or within a block
	* @param	string	$trace				log | screen		Set the output stream to file or screen
	*/
	function trace( 
		$str_message, 
		$str_display = 'inline' ) {

		if ($this->traceEnabled == true) {
			switch ($this->format) {
					case 'html': 
						$CR = $this->CR_html;
						break;
					case 'text':
						$CR = $this->CR_text;
						break;
					default:
						break;
			}

			switch($str_display) {
				case 'inline':
					$str_message = $str_message.$CR;
					break;
				case 'block':
					$str_message =	$CR.$str_message.$CR;
					break;
				default:
					break;
			}

			switch($this->mode) {
				case 'log':
					$fp_log = fopen($this->str_file,'a');
					fputs($fp_log, '['.date("Y-m-d H:i:s").'] '.$str_message);
					fclose($fp_log); 
					break;
				case 'screen':
					print '['.date("Y-m-d H:i:s").'] '.$str_message;
					flush();
					break;
				default:
					break;
			}
		}
	}


	
	
	/**
	* Dumps objects and arrays.
	* Use this function to get an idea of the internal datastructures used. 
	* The function dumps arrays and objects. It renders the content in  an HTML table. 
	* Play with it, you'll see it's very helpful for debugging.
	* @param	string		$title				Optional title used in the HTML Table
	* @param	mixed		$data				Optional array or object that you want to dump. Fallback to $this.
	* @param	boolean	$userfunction	Optional flag. If set to false userfunction in an object are not shown (default). If set to true, userfunctions are rendered
	*
	* @access	public
	*/
	function introspection( $title='Introspection', $data = "", $userfunction = true) {
			if (""==$data) {
				$data = $this;
			}
			
			print('<table border="1" cellspacing="1" cellpadding="1" style="border: 1px solid Black;" width="100%">');	
							
			if ($title != '') {
				printf('<caption style="font-size: 20px; color: Blue; text-align: left;">%s</caption>', $title );
			}
			
			print('<th style="border-bottom: 1px solid Silver;" width="15%">Element</th>');
			print('<th style="border-bottom: 1px solid Silver;" width="8%">Type</th>');
			print('<th style="border-bottom: 1px solid Silver;">Value</th>');

			reset($data);
			while ( list($k,$v) = each($data) ) {
				if ('user function'==gettype($v) && !$userfunction) {
					continue;
				}
				
				if (is_array($v) || is_object($v)) {				
					print('<tr>');
					printf('<td>%s</td>',$k);
					@printf('<td colspan="2">[%s]', @str_replace("<", "&lt;", $v)	);
						$this->introspection("", $v, $userfunction);					
					printf('</td></tr>');
				} else {			
					echo '<tr>';
						printf('<td>%s</td>', $k);
						printf( '<td>[%s]</td>', gettype($v) );
						printf('<td>"%s"</td>', @str_replace("<", "&lt;", $v) );
					echo '</tr>';
				}
			}
			print '</table>';
	}

	/**
	 * Is the string $str_ defined
	 * From what I know, yet no PHP function allows to test if a string is defined.
	 * Defined here means is set and has a positive length
	 *
	 * @since	15/01/2003	BPO	SFFR#668458	eDFramework :: Null & Empty String Test 
	 * @param	string	$str_	a string
	 * @return	TRUE is the string is defined, else FALSE
	 */
	 function is_defined( $str_ ) {
		 if ( isset( $str_) && strlen($str_) ) {
			 return TRUE;
		 } else {
			 return FALSE;
		 }
	 }
}
?>