<?php
// $Id: eDValidator.php,v 1.1.1.1 2002/12/30 13:49:11 bpotier Exp $
/**
  * Validator Class
  */

class Validator {
	var $ERROR	=	"";
	var $CLEAR	=	false;

	/** Constructor */
	function Validator ()	{
		return;
	}

	/** */
	function clear_error () 	{
		$this->ERROR = "";
	}

	/** Checks a string for whitespace. True or false */
	function has_space ($text)	{
		if( ereg("[ 	]",$text) ) {
			return true;
		}
		return false;		
	}

	/** */
	function chconvert ($fragment)	{
		if      ($fragment == 7) { 
			$result = "rwx"; 
		} elseif  ($fragment == 6) { 
			$result = "rw-"; 
		} elseif  ($fragment == 5) { 
			$result = "r-x"; 
		} elseif  ($fragment == 4) { 
			$result = "r--"; 
		} elseif  ($fragment == 3) { 
			$result = "-wx"; 
		} elseif  ($fragment == 2) { 
			$result = "-w-"; 
		} elseif  ($fragment == 1) { 
			$result = "--x"; 
		} elseif  ($fragment == 0) { 
			$result = "---"; 
		} else	{ $result = "unk"; }
		return($result);
    }

	/** */
	function get_perms ($fileName ) {
		if($this->CLEAR) { $this->clear_error(); }
		$atrib = array();
		$perms = fileperms($fileName);
		if(!$perms) 	{
			$this->ERROR = "get_perms: Unable to obtain file perms on [$fileName]";
			return false;
		}
		$octal = sprintf("%lo", ($perms & 07777) );
		$one = substr($octal,0,1);
		$two = substr($octal,1,1);
		$three = substr($octal,2,1);

		$user  = $this->chconvert($one);
		$group = $this->chconvert($two);
		$other = $this->chconvert($three);

		if(is_dir($fileName)) {
			$user = "d$user";
		}

		$atrib = array(
						"octal"	=>	$octal,
						"user"	=>	$user,
						"group"	=>	$group,
						"other"	=>	$other
					);

		return $atrib;
	}

	/** */
	function is_sane ($filename) {
		if($this->CLEAR) { $this->clear_error(); }
		if (!file_exists($filename)) {
			$this->ERROR = "File does not exist";
			return false;
		}
		if (!is_readable($filename)) {
			$this->ERROR = "File is not readable";
			return false;
		}
		if(!is_writeable($filename))
		{
			$this->ERROR = "File is not writeable";
			return false;
		}
		if(is_dir($filename))
		{
			$this->ERROR = "File is a directory";
			return false;
		}
		if(is_link($filename))
		{
			$this->ERROR = "File is a symlink";
			return false;
		}

		return true;
	}

	/** */
	function strip_space ($text) {
		$Return = ereg_replace("([ 	]+)","",$text);
		return ($Return);
	}

	/** */
	function is_allnumbers ($text) {
		if( (gettype($text)) == "integer")	{ return true; }

		$Bad = $this->strip_numbers($text);

		if(empty($Bad))
		{
			return true;
		}
		return false;
	}

	/** */
	function strip_numbers ($text) {
		$Stripped = eregi_replace("([0-9]+)","",$text);
		return ($Stripped);
	}

	/** */
	function is_allletters ($text) {
		$Bad = $this->strip_letters($text);
		if(empty($Bad))
		{
			return true;
		}

		return false;
	}


	/** */
	function strip_letters ($text)	{
		$Stripped = eregi_replace("([A-Z]+)","",$text);
		return $Stripped;
	}

	/**
	  * Checks for HTML entities in submitted text
	  * If found returns true, otherwise false. HTML specials are:
	  *	"	=>	&quot;
	  *	<	=>	&lt;
	  *	>	=>	&gt;
	  *	&	=>	&amp;
	  *	The presence of ",<,>,&  will force this method to return true.
	  */
	  function has_html ($text = "") {
		  if(empty($text)) {
			  return false;
		  }
		  $New = htmlspecialchars($text);
		  if($New == $text) {
			  return false;
			}
			return true;
		}

		/**
		  * strip_html()
		  * Strips all html entities, attributes, elements and tags from
		  * the submitted string data and returns the results.
		  * Can't use a regex here because there's no way to know
		  * how the data is laid out. We have to examine every character
		  * that's been submitted. Consequently, this is not a very
		  * efficient method. It works, it's very good at removing
		  * all html from the data, but don't send gobs of data
		  * at it or your program will slow to a crawl.
		  * If you're stripping HTML from a file, use PHP's fgetss()
		  * and NOT this method, as fgetss() does the same thing
		  * about 100x faster
		  */
		function strip_html ($text = "") {
			if( (!$text) or (empty($text)) ) {
				return "";
			}
			$outside = true;
			$rawText = "";
			$length = strlen($text);
			$count = 0;
			
			for($count=0; $count < $length; $count++) {
				$digit = substr($text,$count,1);
				if(!empty($digit)) {
					if( ($outside) and ($digit != "<") and ($digit != ">") ) 	{
						$rawText .= $digit;
					}
					if($digit == "<") 	{
						$outside = false;
					}
					if($digit == ">") {
						$outside = true;
					}
				}	
			}
			return $rawText;
		}

		/**
		 *	Returns true of the submitted text has meta characters in it
		 *	. \\ + * ? [ ^ ] ( $ )
		 */ 
		function has_metas ($text = "") {
			 if(empty($text)) {
				 return false;
			 }
			 $New = quotemeta($text);
			 if($New == $text) {
				 return false;
			}
			return true;
		}


		/**
		 *	Strips "  . \\ + * ? [ ^ ] ( $ )  " from submitted string
		 * Metas are a virtual MINE FIELD for regular expressions,
		 *  see custom_strip() for how they are removed
		 */
		function strip_metas ($text = "") {
			if(empty($text)) {
				return false;
			}
			$Metas = array( '.','+','*','?','[','^',']','(','$',')' );
			$text = stripslashes($text);
			$New = $this->custom_strip($Metas,$text);
			return $New;
		}

		/**
		 *	$Chars must be an array of characters to remove.
		 *	This method is meta-character safe.
		 */
		 function custom_strip ($Chars, $text = "") {
			 if($this->CLEAR) { $this->clear_error(); }
			 if(empty($text)) {
				 return false;
			}
			if( (gettype($Chars)) != "array") {
				$this->ERROR = "custom_strip: [$Chars] is not an array";
				return false;
			}
			while ( list ( $key,$val) = each ($Chars) ) {
				if(!empty($val)) {
					// str_replace is meta-safe, ereg_replace is not
					$text = str_replace($val,"",$text);
				}
			}
			return $text;
		}

	
	/**
	 *	Array_Echo will walk through an array,
	 *	continuously printing out key value pairs.
	 * Multi dimensional arrays are handled recursively.
	 */
	 function array_echo ($MyArray, $Name = "Array") {
		 if($this->CLEAR) { $this->clear_error(); }
		 if( (gettype($MyArray)) != "array") { return; }
		 $count = 0;
		 
		while ( list ($key,$val) = each ($MyArray) ) {
			if($count == 0) {
				echo "\n\n<P><TABLE BORDER=1 CELLPADDING=0 CELLSPACING=0 COLS=8\n";
				echo "><TR><TD VALIGN=TOP COLSPAN=4><B>$Name Contents:</B></TD\n";
				echo "><TD COLSPAN=2><B>KEY</B></TD><TD COLSPAN=2><B>VAL</B></TD></TR\n>";
			}
			if( (gettype($val)) == "array") {
				$NewName = "$key [$Name $count]";
				$NewArray = $MyArray[$key];
				echo "</TD></TR></TABLE\n\n>";
				$this->array_echo($NewArray,$NewName);
				echo "\n\n<P><TABLE BORDER=1 CELLPADDING=0 CELLSPACING=0 COLS=8\n";
				echo "><TR><TD VALIGN=TOP COLSPAN=4><B>$Name Continued:</B></TD\n";
				echo "><TD COLSPAN=2><B>KEY</B></TD><TD COLSPAN=2><B>VAL</B></TD></TR\n>";
			} else {
				echo "<TR>";
				$Col1 = sprintf("[%s][%0d]",$Name,$count);
				$Col2 = $key;
				if(empty($val))	{ $val = '&nbsp;'; }
				$Col3 = $val;
				echo "<TD COLSPAN=4>$Col1</TD>";
				echo "<TD COLSPAN=2>$Col2</TD\n>";
				echo "<TD COLSPAN=2>$Col3</TD></TR\n\n>";
			}
			$count++;
		}
		echo "<TR><TD COLSPAN=8><B>Array [$Name] complete.</B></TD></TR\n>";
		echo "</TD></TR></TABLE\n\n>";
		return;
	}


	/**
	 * Valid email format? true or false
	 * This checks the raw address, not RFC 822 addresses.
	 * Looks for [something]@[valid hostname with DNS record]
	 */
	function is_email ($Address = "") {
		if($this->CLEAR) { $this->clear_error(); }

		if(empty($Address)) {
			$this->ERROR = "is_email: No email address submitted";
			return false;
		}

		if(!ereg("@",$Address)) {
			$this->ERROR = "is_email: Invalid, no @ symbol in string";
			return false;
		}
		
		list($User,$Host) = split("@",$Address);
		if ( (empty($User)) or (empty($Address)) ) {
			$this->ERROR = "is_email: missing data [$User]@[$Host]";
			return false;
		}
		if( ($this->has_space($User)) or ($this->has_space($Host)) ) {
			$this->ERROR = "is_email: Whitespace in [$User]@[$Host]";
			return false;
		}
		if( !$this->is_host($Host,'ANY') )		{ return false; }
		return true;
	}

	/**
	  * Hostname is a reachable internet host? true or false
	  */
	function is_host ($hostname = '', $type = 'ANY') {
		if ($this->CLEAR) { $this->clear_error(); }
		if ( empty($hostname) ) {
			$this->ERROR = 'is_host: No hostname submitted';
			return false;
		}

		if ( !$this->is_hostname($hostname) )	{ return false; }

		return true;
	}

	/**
	 *	Valid, fully qualified hostname? true or false
	 *	Checks the -syntax- of the hostname, not it's actual
	 * validity as a reachable internet host
	 */
	function is_hostname ($hostname = "") {
		if($this->CLEAR) { $this->clear_error(); }
		$web = false;
		if(empty($hostname)) {
			$this->ERROR = "is_hostname: No hostname submitted";
			return false;
		}

		// Only a-z, 0-9, and "-" or "." are permitted in a hostname
		// Patch for POSIX regex lib by Sascha Schumann sas@schell.de
		$Bad = eregi_replace("[-A-Z0-9\.]","",$hostname);
		if(!empty($Bad)) {
			$this->ERROR = "is_hostname: invalid chars [$Bad]";
			return false;
		}

		// See if we're doing www.hostname.tld or hostname.tld
		if(eregi("^www\.",$hostname)) { $web = true; }

		// double "." is a not permitted
		if(ereg("\.\.",$hostname)) {
			$this->ERROR = "is_hostname: Double dot in [$hostname]";
			return false;
		}
		if(ereg("^\.",$hostname)) 	{
			$this->ERROR = "is_hostname: leading dot in [$hostname]";
			return false;
		}

		$chunks = explode(".",$hostname);
		if( (gettype($chunks)) != "array") {
			$this->ERROR = "is_hostname: Invalid hostname, no dot seperator [$hostname]";
			return false;
		}

		$count = ( (count($chunks)) - 1);

		if($count < 1) 	{
			$this->ERROR = "is_hostname: Invalid hostname [$count] [$hostname]\n";
			return false;
		}

		if( ($web) and ($count < 2) ) {
			$this->ERROR = "is_hostname: Invalid hostname [$count] [$hostname]\n";
			return false;
		}

		$tld = $chunks[$count];

		if(empty($tld)) {
			$this->ERROR = "is_hostname: No TLD found in [$hostname]";
			return false;
		}

		return true;
	}
}
?>